/******************************************************************************
 *
 * Classes for easy reading & writing of WAV sound files.
 *
 * Many thanks to Manish Bajpai whose WAV writing routines I've used as
 * base of these classes.
 *
 * Author        : Copyright (c) Olli Parviainen 2002
 * Author e-mail : oparviai@iki.fi
 * File created  : 13-Jan-2002
 * Last modified : 21-Feb-2002
 *
 * License :
 *
 *  This file is part of SoundTouch sound processing library.
 *
 *  SoundTouch is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  SoundTouch is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SoundTouch; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#ifndef WAVFILE_H
#define WAVFILE_H

#include <stdio.h>

#ifndef uint
typedef unsigned int uint;
#endif           


typedef struct {
    char riff_char[4];
    int package_len;
    char wave[4];
} WavRiff;

typedef struct {
    char fmt[4];
    int format_len;
    short int fixed;
    short int channel_number;
    int sample_rate;
    int byte_rate;
    short int byte_per_sample;
    short int bits_per_sample;
} WavFormat;

typedef struct {
    char data_field[4];
    uint data_len;
} WavData;

typedef struct {
    WavRiff riff;
    WavFormat format;
    WavData data;
} WavHeader;



class WavInFile
{

private:
    FILE *fptr;
    uint dataRead;
    WavHeader header;

    int readWavHeaders();
    int checkCharTags();
    int readHeaderBlock();
    int readRIFFBlock();

public:
    WavInFile(const char *filename);
    ~WavInFile();

    void close();
    void rewind();
    uint getSampleRate() const;
    uint getNumBits() const;
    uint getDataSizeInBytes() const;
    uint getNumSamples() const;
    uint getBytesPerSample() const;
    uint getNumChannels() const;
    uint getLengthMS() const;

    int read(void *buffer, const int bufferSizeInBytes);
    int eof() const;
};


class WavOutFile
{
private:
    FILE *fptr;
    WavHeader header;
    int bytesWritten;

    void fillInHeader(const uint sampleRate, const uint bits, const uint channels);
    void finishHeader();
    void writeHeader();

public:
    WavOutFile(const char *fileName, const int sampleRate, const int bits, const int channels);
    ~WavOutFile();

    void write(const void *buffer, const int numBytes);
    void close();
};

#endif

